@@ -42,14 +42,13 @@
 #define BSON_MAXKEY    0x7F
 
 #define GROW_SLOWLY 1048576
-#define INVALID_STRING_LEN(len) (len < 0 || len > (64*1024*1024))
 
 #define CREATE_BUF_STATIC(n) char b[n];         \
 	buf.start = buf.pos = b;                    \
 	buf.end = b+n;
 
 /* driver */
-int php_mongo_serialize_element(const char*, zval**, buffer*, int TSRMLS_DC);
+int php_mongo_serialize_element(const char* name, int name_len, zval**, buffer*, int TSRMLS_DC);
 
 /* objects */
 void php_mongo_serialize_date(buffer*, zval* TSRMLS_DC);
