@@ -59,9 +59,12 @@ PHP_METHOD(MongoCollection, __construct)
 		return;
 	}
 
-	/* check for empty collection name */
-	if (name_len == 0) {
-		zend_throw_exception_ex(zend_exception_get_default(TSRMLS_C), 0 TSRMLS_CC, "MongoDB::__construct(): invalid name %s", name_str);
+	/* check for empty and invalid collection names */
+	if (
+		name_len == 0 ||
+		memchr(name_str, '\0', name_len) != 0
+	) {
+		zend_throw_exception_ex(mongo_ce_Exception, 2 TSRMLS_CC, "MongoDB::__construct(): invalid name %s", name_str);
 		return;
 	}
 
@@ -410,6 +413,7 @@ static zval* append_getlasterror(zval *coll, buffer *buf, zval *options, mongo_c
 	if (EG(exception)) {
 		zval_ptr_dtor(&cursor_z);
 		zval_ptr_dtor(&cmd_ns_z);
+		zval_ptr_dtor(&cmd);
 		return 0;
 	}
 
@@ -898,7 +902,7 @@ PHP_METHOD(MongoCollection, findAndModify)
    returned. */
 PHP_METHOD(MongoCollection, update)
 {
-	zval *criteria, *newobj, *options = 0;
+	zval *criteria, *newobj, *options = NULL;
 	mongo_collection *c;
 	mongo_connection *connection;
 	buffer buf;
@@ -1304,7 +1308,7 @@ PHP_METHOD(MongoCollection, count)
    document is not empty. */
 PHP_METHOD(MongoCollection, save)
 {
-	zval *a, *options = 0;
+	zval *a, *options = NULL;
 	zval **id;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z|a/", &a, &options) == FAILURE) {
@@ -1315,6 +1319,8 @@ PHP_METHOD(MongoCollection, save)
 	if (!options) {
 		MAKE_STD_ZVAL(options);
 		array_init(options);
+	} else {
+		Z_ADDREF_P(options);
 	}
 
 	if (zend_hash_find(HASH_P(a), "_id", 4, (void**)&id) == SUCCESS) {
@@ -1327,6 +1333,7 @@ PHP_METHOD(MongoCollection, save)
 
 		add_assoc_bool(options, "upsert", 1);
 
+		Z_ADDREF_P(options);
 		MONGO_METHOD3(MongoCollection, update, return_value, getThis(), criteria, a, options);
 
 		zval_ptr_dtor(&criteria);
