@@ -468,10 +468,11 @@ int mongo_connection_ismaster(mongo_con_manager *manager, mongo_connection *con,
 	char          *data_buffer;
 	char          *set = NULL;      /* For replicaset in return */
 	char          *hosts, *passives = NULL, *ptr, *string;
-	unsigned char  ismaster = 0, secondary = 0, arbiter = 0;
 	char          *connected_name, *we_think_we_are;
 	struct timeval now;
 	int            retval = 1;
+	unsigned char  ismaster = 0, secondary = 0, arbiter = 0;
+	int original_connection_type = 0;
 
 	gettimeofday(&now, NULL);
 	if ((con->last_ismaster + manager->ismaster_interval) > now.tv_sec) {
@@ -558,7 +559,32 @@ int mongo_connection_ismaster(mongo_con_manager *manager, mongo_connection *con,
 	bson_find_field_as_bool(ptr, "secondary", &secondary);
 	bson_find_field_as_bool(ptr, "arbiterOnly", &arbiter);
 
-	mongo_manager_log(manager, MLOG_CON, MLOG_INFO, "ismaster: set name: %s, ismaster: %d, secondary: %d, is_arbiter: %d", set, ismaster, secondary, arbiter);
+	original_connection_type = con->connection_type;
+	/* Set connection type depending on flags */
+	if (ismaster) {
+		char *msg;
+
+		/* Find msg and whether it contains "isdbgrid" */
+		if (bson_find_field_as_string(ptr, "msg", (char**) &msg) && strcmp(msg, "isdbgrid") == 0) {
+			con->connection_type = MONGO_NODE_MONGOS;
+		} else if (bson_find_field_as_string(ptr, "setName", &set)) {
+			con->connection_type = MONGO_NODE_PRIMARY;
+		} else {
+			con->connection_type = MONGO_NODE_STANDALONE;
+		}
+	} else if (secondary) {
+		con->connection_type = MONGO_NODE_SECONDARY;
+	} else if (arbiter) {
+		con->connection_type = MONGO_NODE_ARBITER;
+	} else {
+		con->connection_type = MONGO_NODE_INVALID;
+	}
+
+	if (con->connection_type != original_connection_type) {
+		*error_message = strdup("The node has changed type, disconnecting");
+		free(data_buffer);
+		return 0;
+	}
 
 	/* Find all hosts */
 	bson_find_field_as_array(ptr, "hosts", &hosts);
@@ -585,17 +611,6 @@ int mongo_connection_ismaster(mongo_con_manager *manager, mongo_connection *con,
 		}
 	}
 
-	/* Set connection type depending on flags */
-	if (ismaster) {
-		con->connection_type = MONGO_NODE_PRIMARY;
-	} else if (secondary) {
-		con->connection_type = MONGO_NODE_SECONDARY;
-	} else if (arbiter) {
-		con->connection_type = MONGO_NODE_ARBITER;
-	} else {
-		con->connection_type = MONGO_NODE_INVALID;
-	} /* TODO: case for mongos */
-
 	free(data_buffer);
 
 	con->last_ismaster = now.tv_sec;
@@ -615,6 +630,8 @@ int mongo_connection_get_server_flags(mongo_con_manager *manager, mongo_connecti
 	char          *ptr;
 	char          *tags;
 	char          *msg; /* If set and its value is "isdbgrid", it signals we connected to a mongos */
+	char          *set = NULL;      /* check if it is a replicaset */
+	unsigned char  ismaster = 0, secondary = 0, arbiter = 0;
 
 	mongo_manager_log(manager, MLOG_CON, MLOG_INFO, "get_server_flags: start");
 	packet = bson_create_ismaster_packet(con);
@@ -648,12 +665,37 @@ int mongo_connection_get_server_flags(mongo_con_manager *manager, mongo_connecti
 		mongo_manager_log(manager, MLOG_CON, MLOG_FINE, "get_server_flags: can't find maxMessageSizeBytes, defaulting to %d", con->max_message_size);
 	}
 
-	/* Find msg and whether it contains "isdbgrid" */
-	if (bson_find_field_as_string(ptr, "msg", (char**) &msg)) {
-		if (strcmp(msg, "isdbgrid") == 0) {
+	/* Check for flags */
+	bson_find_field_as_bool(ptr, "ismaster", &ismaster);
+	bson_find_field_as_bool(ptr, "secondary", &secondary);
+	bson_find_field_as_bool(ptr, "arbiterOnly", &arbiter);
+
+	/* Set connection type depending on flags */
+	if (ismaster) {
+		/* Find msg and whether it contains "isdbgrid" */
+		if (bson_find_field_as_string(ptr, "msg", (char**) &msg) && strcmp(msg, "isdbgrid") == 0) {
 			mongo_manager_log(manager, MLOG_CON, MLOG_FINE, "get_server_flags: msg contains 'isdbgrid' - we're connected to a mongos");
 			con->connection_type = MONGO_NODE_MONGOS;
+		} else if (bson_find_field_as_string(ptr, "setName", &set)) {
+			mongo_manager_log(manager, MLOG_CON, MLOG_FINE, "get_server_flags: got replicaset name: %s", set);
+			con->connection_type = MONGO_NODE_PRIMARY;
+		} else {
+			con->connection_type = MONGO_NODE_STANDALONE;
 		}
+	} else if (secondary) {
+		con->connection_type = MONGO_NODE_SECONDARY;
+	} else if (arbiter) {
+		con->connection_type = MONGO_NODE_ARBITER;
+	} else {
+		con->connection_type = MONGO_NODE_INVALID;
+	}
+
+	mongo_manager_log(manager, MLOG_CON, MLOG_INFO, "get_server_flags: found server type: %s", mongo_connection_type(con->connection_type));
+
+	if (con->connection_type == MONGO_NODE_INVALID) {
+		*error_message = strdup("get_server_flags: got unknown node type");
+		free(data_buffer);
+		return 0;
 	}
 
 	/* Find read preferences tags */
