@@ -29,15 +29,62 @@
 #include <stdio.h>
 #include "types.h"
 
+#ifndef WIN32
+#include <poll.h>
 
 /* Wait on socket availability with a timeout
- * TODO: Port to use poll() instead of select().
  *
  * Returns:
  * 0 on success
  * -1 on failure, but not critical enough to throw an exception
  * 1.. on failure, and throw an exception. The return value is the error code */
 int mongo_io_wait_with_timeout(int sock, int to, char **error_message)
+{
+	/* No socket timeout.. But we default to 1 second for historical reasons */
+	if (to < 1) {
+		to = 1000;
+	}
+	while (1) {
+		struct pollfd fds[1];
+		int status;
+
+		fds[0].fd = sock;
+		fds[0].events = POLLIN | POLLHUP | POLLERR;
+		status = poll(fds, 1, to);
+
+		if (status == -1) {
+			/* on EINTR, retry - this resets the timeout however to its full length */
+			if (errno == EINTR) {
+				continue;
+			}
+
+			*error_message = strdup(strerror(errno));
+			return 13;
+		}
+
+		if (status == 0) {
+			*error_message = malloc(256);
+			snprintf(
+				*error_message, 256,
+				"cursor timed out (timeout: %d, status: %d)",
+				to, status
+			);
+			return 80;
+		}
+
+		if (status > 0 && !(fds[0].revents & POLLIN)) {
+			*error_message = strdup("Exceptional condition on socket");
+			return 17;
+		}
+		break;
+	}
+
+	return 0;
+}
+#else
+
+/* Win32 doesn't have a working poll(), so we use the old select() branch. */
+int mongo_io_wait_with_timeout(int sock, int to, char **error_message)
 {
 	/* No socket timeout.. But we default to 1 second for historical reasons */
 	if (to < 1) {
@@ -91,6 +138,7 @@ int mongo_io_wait_with_timeout(int sock, int to, char **error_message)
 
 	return 0;
 }
+#endif
 
 /* Low-level send function.
  *
@@ -139,9 +187,17 @@ int mongo_io_recv_header(mongo_connection *con, mongo_server_options *options, i
 
 	if (status == -1) {
 		*error_message = strdup(strerror(errno));
+#ifndef WIN32
+		if (errno == ECONNRESET) {
+			return -32;
+		} else {
+			return -31;
+		}
+#else
 		return -31;
+#endif
 	} else if (status == 0) {
-		*error_message = strdup("The socket was closed by remote host");
+		*error_message = strdup("Remote server has closed the connection");
 		return -32;
 	}
 	return status;
