@@ -112,13 +112,17 @@ static mongo_connection *mongo_get_connection_single(mongo_con_manager *manager,
 	if (con) {
 		/* Do authentication if requested */
 		if (server->db && server->username && server->password) {
-			mongo_manager_log(manager, MLOG_CON, MLOG_INFO, "get_connection_single: authenticating %s", hash);
-			if (!authenticate_connection(manager, con, options, server->authdb ? server->authdb : server->db, server->username, server->password, error_message)) {
-				mongo_connection_destroy(manager, con, MONGO_CLOSE_BROKEN);
-				free(hash);
-				return NULL;
+			/* Note: Arbiters don't contain any data, including auth stuff, so you cannot authenticate on an arbiter */
+			if (con->connection_type != MONGO_NODE_ARBITER) {
+				mongo_manager_log(manager, MLOG_CON, MLOG_INFO, "get_connection_single: authenticating %s", hash);
+				if (!authenticate_connection(manager, con, options, server->authdb ? server->authdb : server->db, server->username, server->password, error_message)) {
+					mongo_connection_destroy(manager, con, MONGO_CLOSE_BROKEN);
+					free(hash);
+					return NULL;
+				}
 			}
 		}
+
 		/* Do the first-time ping to record the latency of the connection */
 		if (mongo_connection_ping(manager, con, options, error_message)) {
 			/* Register the connection on successful pinging */
@@ -410,6 +414,12 @@ int mongo_deregister_callback_from_connection(mongo_connection *connection, void
 /* API interface to fetch a connection */
 mongo_connection *mongo_get_read_write_connection(mongo_con_manager *manager, mongo_servers *servers, int connection_flags, char **error_message)
 {
+	/* In some cases we won't actually have a manager or servers initialized, for example when extending PHP objects without calling the constructor,
+	 * and then var_dump() it or access the properties, for example the "connected" property */
+	if (!manager || !servers) {
+		return NULL;
+	}
+
 	/* Which connection we return depends on the type of connection we want */
 	switch (servers->options.con_type) {
 		case MONGO_CON_TYPE_STANDALONE:
@@ -505,6 +515,15 @@ void *mongo_manager_find_by_hash(mongo_con_manager *manager, mongo_con_manager_i
 	return NULL;
 }
 
+mongo_connection *mongo_manager_connection_find_by_server_definition(mongo_con_manager *manager, mongo_server_def *definition)
+{
+	char *hash = mongo_server_create_hash(definition);
+	mongo_connection *con = mongo_manager_find_by_hash(manager, manager->connections, hash);
+
+	free(hash);
+	return con;
+}
+
 mongo_connection *mongo_manager_connection_find_by_hash(mongo_con_manager *manager, char *hash)
 {
 	return mongo_manager_find_by_hash(manager, manager->connections, hash);
