@@ -1,5 +1,5 @@
 /**
- *  Copyright 2009-2012 10gen, Inc.
+ *  Copyright 2009-2013 10gen, Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -44,6 +44,10 @@ mongo_servers* mongo_parse_init(void)
 	servers->options.default_w = -1;
 	servers->options.default_wstring = NULL;
 	servers->options.default_wtimeout = -1;
+	servers->options.default_fsync = 0;
+	servers->options.default_journal = 0;
+	servers->options.ssl = MONGO_SSL_DISABLE;
+	servers->options.ctx = NULL;
 
 	return servers;
 }
@@ -125,8 +129,9 @@ int mongo_parse_server_spec(mongo_con_manager *manager, mongo_servers *servers,
 			pos++;
 		} while (*pos != '\0');
 
-		/* We are now either at the end of the string, or at / where the dbname starts.
-		 * We still have to add the last parser host/port combination though: */
+		/* We are now either at the end of the string, or at / where the dbname
+		 * starts.  We still have to add the last parser host/port combination
+		 * though: */
 		mongo_add_parsed_server_addr(manager, servers, host_start, host_end ? host_end : pos, port_start);
 	} else if (*pos == '/') {
 		host_start = pos;
@@ -139,8 +144,9 @@ int mongo_parse_server_spec(mongo_con_manager *manager, mongo_servers *servers,
 		 */
 		last_slash = strrchr(pos, '/');
 
-		/* The last component of the path *could* be a database name.
-		 * The rule is; if the last component has a dot, we use the full string since "host_start" as host */
+		/* The last component of the path *could* be a database name.  The rule
+		 * is; if the last component has a dot, we use the full string since
+		 * "host_start" as host */
 		if (strchr(last_slash, '.')) {
 			host_end = host_start + strlen(host_start);
 		} else {
@@ -227,7 +233,8 @@ void static mongo_add_parsed_server_addr(mongo_con_manager *manager, mongo_serve
 
 	tmp = malloc(sizeof(mongo_server_def));
 	memset(tmp, 0, sizeof(mongo_server_def));
-	tmp->username = tmp->password = tmp->db = NULL;
+	tmp->username = tmp->password = tmp->db = tmp->authdb = NULL;
+	tmp->mechanism = MONGO_AUTH_MECHANISM_MONGODB_CR; /* MONGODB-CR is the default authentication mechanism */
 	tmp->port = 27017;
 
 	tmp->host = mcon_strndup(host_start, host_end - host_start);
@@ -240,8 +247,12 @@ void static mongo_add_parsed_server_addr(mongo_con_manager *manager, mongo_serve
 }
 
 /* Processes a single option/value pair.
- * Returns 0 if it worked, 1 if either name or value was missing, 2 if the option didn't exist, 3 on logic errors
- */
+ * Returns:
+ * -1 if it worked, but the option really shouldn't be used
+ * 0 if it worked
+ * 1 if either name or value was missing
+ * 2 if the option didn't exist
+ * 3 on logic errors */
 int static mongo_process_option(mongo_con_manager *manager, mongo_servers *servers, char *name, char *value, char *pos, char **error_message)
 {
 	char *tmp_name;
@@ -287,8 +298,10 @@ static int parse_read_preference_tags(mongo_con_manager *manager, mongo_servers
 			end = strchr(start, ',');
 			colon = strchr(start, ':');
 			if (!colon) {
-				*error_message = malloc(256 + strlen(start));
-				snprintf(*error_message, 256 + strlen(start), "Error while trying to parse tags: No separator for '%s'", start);
+				int len = strlen(start) + sizeof("Error while trying to parse tags: No separator for ''");
+
+				*error_message = malloc(len + 1);
+				snprintf(*error_message, len, "Error while trying to parse tags: No separator for '%s'", start);
 				mongo_read_preference_tagset_dtor(tmp_ts);
 				return 3;
 			}
@@ -313,59 +326,96 @@ static int parse_read_preference_tags(mongo_con_manager *manager, mongo_servers
 }
 
 /* Sets server options.
- * Returns 0 if it worked, 2 if the option didn't exist, 3 on logical errors.
- * On logical errors, the error_message will be populated with the reason.
- */
+ * Returns:
+ * 0 if it worked
+ * 2 if the option didn't exist
+ * 3 on logical errors.
+ *
+ * On logical errors, the error_message will be populated with the reason. */
 int mongo_store_option(mongo_con_manager *manager, mongo_servers *servers, char *option_name, char *option_value, char **error_message)
 {
 	int i;
 
-	if (strcasecmp(option_name, "replicaSet") == 0) {
-		if (servers->options.repl_set_name) {
-			/* Free the already existing one */
-			free(servers->options.repl_set_name);
-			servers->options.repl_set_name = NULL; /* We reset it as not all options set a string as replset name */
-		}
+	if (strcasecmp(option_name, "authMechanism") == 0) {
+		int mechanism;
 
-		if (option_value && *option_value) {
-			/* We explicitly check for the stringified version of "true" here,
-			 * as "true" has a special meaning. It does not mean that the
-			 * replicaSet name is "1". */
-			if (strcmp(option_value, "1") != 0) {
-				servers->options.repl_set_name = strdup(option_value);
-				mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'replicaSet': '%s'", option_value);
+		mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'authMechanism': '%s'", option_value);
+		if (strcasecmp(option_value, "MONGODB-CR") == 0) {
+			mechanism = MONGO_AUTH_MECHANISM_MONGODB_CR;
+		} else if (strcasecmp(option_value, "GSSAPI") == 0) {
+			/* FIXME: GSSAPI isn't implemented yet */
+			mechanism = MONGO_AUTH_MECHANISM_GSSAPI;
+			*error_message = strdup("The authMechanism 'GSSAPI' is currently not supported. Only MONGODB-CR is available.");
+			return 3;
+		} else {
+			int len = strlen(option_value) + sizeof("The authMechanism '' does not exist.");
 
-				/* Associate the given replica set name with all the server definitions from the seed */
-				for (i = 0; i < servers->count; i++) {
-					if (servers->server[i]->repl_set_name) {
-						free(servers->server[i]->repl_set_name);
-					}
-					servers->server[i]->repl_set_name = strdup(option_value);
-				}
-			} else {
-				mongo_manager_log(manager, MLOG_PARSE, MLOG_WARN, "- Found option 'replicaSet': true - Expected the name of the replica set");
-			}
-			servers->options.con_type = MONGO_CON_TYPE_REPLSET;
-			mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Switching connection type: REPLSET");
+			*error_message = malloc(len + 1);
+			snprintf(*error_message, len, "The authMechanism '%s' does not exist.", option_value);
+			return 3;
 		}
-		return 0;
-	}
-	if (strcasecmp(option_name, "username") == 0) {
-		mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'username': '%s'", option_value);
 		for (i = 0; i < servers->count; i++) {
-			if (servers->server[i]->username) {
-				free(servers->server[i]->username);
-			}
-			servers->server[i]->username = strdup(option_value);
-			/* Use "admin" as the default db if none selected yet. It is okay
-			 * if it is set in a later option, as we first always free the
-			 * value before setting it anyway. */
-			if (!servers->server[i]->db) {
-				servers->server[i]->db = strdup("admin");
-			}
+			servers->server[i]->mechanism = mechanism;
 		}
 		return 0;
 	}
+
+	if (strcasecmp(option_name, "authSource") == 0) {
+		mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'authSource': '%s'", option_value);
+		for (i = 0; i < servers->count; i++) {
+			if (servers->server[i]->authdb) {
+				free(servers->server[i]->authdb);
+			}
+			servers->server[i]->authdb = strdup(option_value);
+		}
+		return 0;
+	}
+
+	if (strcasecmp(option_name, "connectTimeoutMS") == 0) {
+		int value = atoi(option_value);
+
+		if (servers->options.connectTimeoutMS) {
+			mongo_manager_log(manager, MLOG_PARSE, MLOG_WARN, "- Replacing previously set value for 'connectTimeoutMS' (%d)", servers->options.connectTimeoutMS);
+		}
+		mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'connectTimeoutMS': %d", value);
+		servers->options.connectTimeoutMS = value;
+		return 0;
+	}
+
+	if (strcasecmp(option_name, "db") == 0) {
+		mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'db': '%s'", option_value);
+		for (i = 0; i < servers->count; i++) {
+			if (servers->server[i]->db) {
+				free(servers->server[i]->db);
+				/* Free the authdb too as it defaulted to 'admin' when no db was passed as the connection string */
+				free(servers->server[i]->authdb);
+				servers->server[i]->authdb = NULL;
+			}
+			servers->server[i]->db = strdup(option_value);
+		}
+		return 0;
+	}
+
+	if (strcasecmp(option_name, "fsync") == 0) {
+		if (strcasecmp(option_value, "true") == 0 || strcmp(option_value, "1") == 0) {
+			servers->options.default_fsync = 1;
+		} else {
+			servers->options.default_fsync = 0;
+		}
+		mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'fsync': %d", servers->options.default_fsync);
+		return 0;
+	}
+
+	if (strcasecmp(option_name, "journal") == 0) {
+		if (strcasecmp(option_value, "true") == 0 || strcmp(option_value, "1") == 0) {
+			servers->options.default_journal = 1;
+		} else {
+			servers->options.default_journal = 0;
+		}
+		mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'journal': %d", servers->options.default_journal);
+		return 0;
+	}
+
 	if (strcasecmp(option_name, "password") == 0) {
 		mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'password': '%s'", option_value);
 		for (i = 0; i < servers->count; i++) {
@@ -376,35 +426,7 @@ int mongo_store_option(mongo_con_manager *manager, mongo_servers *servers, char
 		}
 		return 0;
 	}
-	if (strcasecmp(option_name, "db") == 0) {
-		mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'db': '%s'", option_value);
-		for (i = 0; i < servers->count; i++) {
-			if (servers->server[i]->db) {
-				free(servers->server[i]->db);
-			}
-			servers->server[i]->db = strdup(option_value);
-		}
-		return 0;
-	}
-	if (strcasecmp(option_name, "slaveOkay") == 0) {
-		if (strcasecmp(option_value, "true") == 0 || *option_value == '1') {
-			mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'slaveOkay': true");
-			if (servers->read_pref.type != MONGO_RP_PRIMARY || servers->read_pref.tagset_count) {
-				/* the server already has read preferences configured, but we're still
-				 * trying to set slave okay. The spec says that's an error */
-				*error_message = strdup("You can not use both slaveOkay and read-preferences. Please switch to read-preferences.");
-				return 3;
-			} else {
-				/* Old style option, that needs to be removed. For now, spec dictates
-				 * it needs to be ReadPreference=SECONDARY_PREFERRED */
-				servers->read_pref.type = MONGO_RP_SECONDARY_PREFERRED;
-			}
-			return 0;
-		}
 
-		mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'slaveOkay': false");
-		return 0;
-	}
 	if (strcasecmp(option_name, "readPreference") == 0) {
 		mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'readPreference': '%s'", option_value);
 		if (strcasecmp(option_value, "primary") == 0) {
@@ -426,31 +448,63 @@ int mongo_store_option(mongo_con_manager *manager, mongo_servers *servers, char
 		}
 		return 0;
 	}
+
 	if (strcasecmp(option_name, "readPreferenceTags") == 0) {
 		mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'readPreferenceTags': '%s'", option_value);
 		return parse_read_preference_tags(manager, servers, option_value, error_message);
 	}
 
-	if (strcasecmp(option_name, "timeout") == 0) {
-		int value = atoi(option_value);
+	if (strcasecmp(option_name, "replicaSet") == 0) {
+		if (servers->options.repl_set_name) {
+			/* Free the already existing one */
+			free(servers->options.repl_set_name);
+			servers->options.repl_set_name = NULL; /* We reset it as not all options set a string as replset name */
+		}
 
-		if (servers->options.connectTimeoutMS) {
-			mongo_manager_log(manager, MLOG_PARSE, MLOG_WARN, "- Replacing previously set value for 'connectTimeoutMS' (%d)", servers->options.connectTimeoutMS);
+		if (option_value && *option_value) {
+			/* We explicitly check for the stringified version of "true" here,
+			 * as "true" has a special meaning. It does not mean that the
+			 * replicaSet name is "1". */
+			if (strcmp(option_value, "1") != 0) {
+				servers->options.repl_set_name = strdup(option_value);
+				mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'replicaSet': '%s'", option_value);
+
+				/* Associate the given replica set name with all the server
+				 * definitions from the seed */
+				for (i = 0; i < servers->count; i++) {
+					if (servers->server[i]->repl_set_name) {
+						free(servers->server[i]->repl_set_name);
+					}
+					servers->server[i]->repl_set_name = strdup(option_value);
+				}
+			} else {
+				mongo_manager_log(manager, MLOG_PARSE, MLOG_WARN, "- Found option 'replicaSet': true - Expected the name of the replica set");
+			}
+			servers->options.con_type = MONGO_CON_TYPE_REPLSET;
+			mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Switching connection type: REPLSET");
 		}
-		mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'timeout' ('connectTimeoutMS'): %d", value);
-		servers->options.connectTimeoutMS = value;
 		return 0;
 	}
 
-	if (strcasecmp(option_name, "connectTimeoutMS") == 0) {
-		int value = atoi(option_value);
-
-		if (servers->options.connectTimeoutMS) {
-			mongo_manager_log(manager, MLOG_PARSE, MLOG_WARN, "- Replacing previously set value for 'connectTimeoutMS' (%d)", servers->options.connectTimeoutMS);
+	if (strcasecmp(option_name, "slaveOkay") == 0) {
+		if (strcasecmp(option_value, "true") == 0 || strcmp(option_value, "1") == 0) {
+			mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'slaveOkay': true");
+			if (servers->read_pref.type != MONGO_RP_PRIMARY || servers->read_pref.tagset_count) {
+				/* The server already has read preferences configured, but
+				 * we're still trying to set slave okay. The spec says that's
+				 * an error */
+				*error_message = strdup("You can not use both slaveOkay and read-preferences. Please switch to read-preferences.");
+				return 3;
+			} else {
+				/* Old style option, that needs to be removed. For now, spec
+				 * dictates it needs to be ReadPreference=SECONDARY_PREFERRED */
+				servers->read_pref.type = MONGO_RP_SECONDARY_PREFERRED;
+			}
+			return -1;
 		}
-		mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'connectTimeoutMS': %d", value);
-		servers->options.connectTimeoutMS = value;
-		return 0;
+
+		mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'slaveOkay': false");
+		return -1;
 	}
 
 	if (strcasecmp(option_name, "socketTimeoutMS") == 0) {
@@ -461,10 +515,72 @@ int mongo_store_option(mongo_con_manager *manager, mongo_servers *servers, char
 		return 0;
 	}
 
+	if (strcasecmp(option_name, "ssl") == 0) {
+		int value = 0;
+		if (strcasecmp(option_value, "true") == 0 || strcmp(option_value, "1") == 0) {
+			value = MONGO_SSL_ENABLE;
+			mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'ssl': true");
+		} else if (strcasecmp(option_value, "false") == 0 || strcmp(option_value, "0") == 0) {
+			value = MONGO_SSL_DISABLE;
+			mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'ssl': false");
+		} else if (strcasecmp(option_value, "prefer") == 0 || atoi(option_value) == MONGO_SSL_PREFER) {
+			/* FIXME: MongoDB doesn't support "connection promotion" to SSL at
+			 * the moment, so we can't support this option properly */
+			value = MONGO_SSL_PREFER;
+			mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'ssl': prefer");
+			*error_message = strdup("SSL=prefer is currently not supported by mongod");
+			return 3;
+		} else {
+			mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'ssl': '%s'", option_name);
+			*error_message = strdup("SSL can only be 'true' or 'false'");
+			return 3;
+		}
+
+		servers->options.ssl = value;
+		return 0;
+	}
+
+	if (strcasecmp(option_name, "timeout") == 0) {
+		int value = atoi(option_value);
+
+		if (servers->options.connectTimeoutMS) {
+			mongo_manager_log(manager, MLOG_PARSE, MLOG_WARN, "- Replacing previously set value for 'connectTimeoutMS' (%d)", servers->options.connectTimeoutMS);
+		}
+		mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'timeout' ('connectTimeoutMS'): %d", value);
+		servers->options.connectTimeoutMS = value;
+		return -1;
+	}
+
+	if (strcasecmp(option_name, "username") == 0) {
+		mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'username': '%s'", option_value);
+		for (i = 0; i < servers->count; i++) {
+			if (servers->server[i]->username) {
+				free(servers->server[i]->username);
+			}
+			servers->server[i]->username = strdup(option_value);
+			/* Use "admin" as the default db if none selected yet. It is okay
+			 * if it is set in a later option, as we first always free the
+			 * value before setting it anyway. */
+			if (!servers->server[i]->db) {
+				servers->server[i]->db = strdup("admin");
+				/* Admin users always authenticate on the admin db, even when
+				 * using other databases */
+				servers->server[i]->authdb = strdup("admin");
+			}
+		}
+		return 0;
+	}
+
 	if (strcasecmp(option_name, "w") == 0) {
 		/* Rough check to see whether this is a numeric string or not */
-		if ((option_value[0] >= '0' && option_value[0] <= '9') || option_value[0] == '-') {
-			servers->options.default_w = atoi(option_value);
+		char *endptr;
+		long tmp_value;
+		
+		tmp_value = strtol(option_value, &endptr, 10);
+		/* If no invalid character is found (endptr == 0), we consider the
+		 * option value as a number */
+		if (!*endptr) {
+			servers->options.default_w = tmp_value;
 			mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO, "- Found option 'w': %d", servers->options.default_w);
 			if (servers->options.default_w < 0) {
 				*error_message = strdup("The value of 'w' needs to be 0 or higher (or a string).");
@@ -539,8 +655,8 @@ int static mongo_parse_options(mongo_con_manager *manager, mongo_servers *server
 void static mongo_server_def_dump(mongo_con_manager *manager, mongo_server_def *server_def)
 {
 	mongo_manager_log(manager, MLOG_PARSE, MLOG_INFO,
-		"- host: %s; port: %d; username: %s, password: %s, database: %s",
-		server_def->host, server_def->port, server_def->username, server_def->password, server_def->db);
+		"- host: %s; port: %d; username: %s, password: %s, database: %s, auth source: %s, mechanism: %d",
+		server_def->host, server_def->port, server_def->username, server_def->password, server_def->db, server_def->authdb, server_def->mechanism);
 }
 
 void mongo_servers_dump(mongo_con_manager *manager, mongo_servers *servers)
@@ -567,7 +683,8 @@ void mongo_servers_dump(mongo_con_manager *manager, mongo_servers *servers)
 /* Cloning */
 static void mongo_server_def_copy(mongo_server_def *to, mongo_server_def *from, int flags)
 {
-	to->host = to->repl_set_name = to->db = to->username = to->password = NULL;
+	to->host = to->repl_set_name = to->db = to->authdb = to->username = to->password = NULL;
+	to->mechanism = MONGO_AUTH_MECHANISM_MONGODB_CR;
 	if (from->host) {
 		to->host = strdup(from->host);
 	}
@@ -580,12 +697,16 @@ static void mongo_server_def_copy(mongo_server_def *to, mongo_server_def *from,
 		if (from->db) {
 			to->db = strdup(from->db);
 		}
+		if (from->authdb) {
+			to->authdb = strdup(from->authdb);
+		}
 		if (from->username) {
 			to->username = strdup(from->username);
 		}
 		if (from->password) {
 			to->password = strdup(from->password);
 		}
+		to->mechanism = from->mechanism;
 	}
 }
 
@@ -612,6 +733,14 @@ void mongo_servers_copy(mongo_servers *to, mongo_servers *from, int flags)
 	if (from->options.default_wstring) {
 		to->options.default_wstring = strdup(from->options.default_wstring);
 	}
+	to->options.default_fsync = from->options.default_fsync;
+	to->options.default_journal = from->options.default_journal;
+
+	to->options.ssl = from->options.ssl;
+
+	if (from->options.ctx) {
+		memcpy(to->options.ctx, from->options.ctx, sizeof(void *));
+	}
 
 	mongo_read_preference_copy(&from->read_pref, &to->read_pref);
 }
@@ -628,6 +757,9 @@ void mongo_server_def_dtor(mongo_server_def *server_def)
 	if (server_def->db) {
 		free(server_def->db);
 	}
+	if (server_def->authdb) {
+		free(server_def->authdb);
+	}
 	if (server_def->username) {
 		free(server_def->username);
 	}
@@ -658,3 +790,12 @@ void mongo_servers_dtor(mongo_servers *servers)
 	}
 	free(servers);
 }
+
+/*
+ * Local variables:
+ * tab-width: 4
+ * c-basic-offset: 4
+ * End:
+ * vim600: fdm=marker
+ * vim: noet sw=4 ts=4
+ */
