@@ -371,7 +371,7 @@ void php_mongo_ctor(INTERNAL_FUNCTION_PARAMETERS, int bc)
 	/* Parse the server specification
 	 * Default to the mongo.default_host & mongo.default_port INI options */
 	link->servers = mongo_parse_init();
-	if (server) {
+	if (server_len) {
 		error = mongo_parse_server_spec(link->manager, link->servers, server, (char **)&error_message);
 		if (error) {
 			zend_throw_exception(mongo_ce_ConnectionException, error_message, 20 + error TSRMLS_CC);
@@ -459,14 +459,24 @@ void php_mongo_ctor(INTERNAL_FUNCTION_PARAMETERS, int bc)
 		return;
 	}
 #endif
-	if (zdoptions) {
-		/* Possibly add more indexes in the future, like "log_queries",
-		 * "log_commands", "log_failures", "log_metadata"... */
-		zval **zcontext;
 
-		if (zend_hash_find(Z_ARRVAL_P(zdoptions), "context", strlen("context") + 1, (void**)&zcontext) == SUCCESS) {
-			link->servers->options.ctx = php_stream_context_from_zval(*zcontext, PHP_FILE_NO_DEFAULT_CONTEXT);
+	{
+		int i = 0;
+		zval **zcontext;
+		php_stream_context *ctx = NULL;
+
+		if (zdoptions && zend_hash_find(Z_ARRVAL_P(zdoptions), "context", strlen("context") + 1, (void**)&zcontext) == SUCCESS) {
 			mongo_manager_log(link->manager, MLOG_CON, MLOG_INFO, "Found Stream context");
+			ctx = php_stream_context_from_zval(*zcontext, PHP_FILE_NO_DEFAULT_CONTEXT);
+		}
+		link->servers->options.ctx = ctx;
+
+		for (i = 0; i < link->servers->count; i++) {
+			mongo_connection *con = mongo_manager_connection_find_by_server_definition(link->manager, link->servers->server[i]);
+
+			if (con) {
+				php_stream_context_set(con->socket, ctx);
+			}
 		}
 	}
 
@@ -619,8 +629,13 @@ PHP_METHOD(MongoClient, selectDB)
 		return;
 	}
 
+	if (memchr(db, '\0', db_len) != NULL) {
+		zend_throw_exception_ex(mongo_ce_Exception, 2 TSRMLS_CC, "'\\0' not allowed in database names: %s\\0...", db);
+		return;
+	}
+
 	MAKE_STD_ZVAL(name);
-	ZVAL_STRING(name, db, 1);
+	ZVAL_STRINGL(name, db, db_len, 1);
 
 	PHP_MONGO_GET_LINK(getThis());
 
@@ -699,7 +714,7 @@ PHP_METHOD(MongoClient, __get)
 	}
 
 	MAKE_STD_ZVAL(name);
-	ZVAL_STRING(name, str, 1);
+	ZVAL_STRINGL(name, str, str_len, 1);
 
 	/* select this db */
 	MONGO_METHOD1(MongoClient, selectDB, return_value, getThis(), name);
@@ -722,7 +737,7 @@ PHP_METHOD(MongoClient, selectCollection)
 	}
 
 	MAKE_STD_ZVAL(db_name);
-	ZVAL_STRING(db_name, db, 1);
+	ZVAL_STRINGL(db_name, db, db_len, 1);
 
 	MAKE_STD_ZVAL(temp_db);
 	MONGO_METHOD1(MongoClient, selectDB, temp_db, getThis(), db_name);
@@ -730,7 +745,7 @@ PHP_METHOD(MongoClient, selectCollection)
 	PHP_MONGO_CHECK_EXCEPTION1(&temp_db);
 
 	MAKE_STD_ZVAL(coll_name);
-	ZVAL_STRING(coll_name, coll, 1);
+	ZVAL_STRINGL(coll_name, coll, coll_len, 1);
 
 	MONGO_METHOD1(MongoDB, selectCollection, return_value, temp_db, coll_name);
 
