@@ -206,8 +206,15 @@ PHP_MINIT_FUNCTION(mongo)
 	}
 #endif
 
+#if MONGO_PHP_STREAMS
+	REGISTER_LONG_CONSTANT("MONGO_STREAMS", 1, CONST_PERSISTENT);
+#else
+	REGISTER_LONG_CONSTANT("MONGO_STREAMS", 0, CONST_PERSISTENT);
+#endif
+
 	return SUCCESS;
 }
+/* }}} */
 
 /* {{{ PHP_GINIT_FUNCTION
  */
@@ -227,7 +234,6 @@ static PHP_GINIT_FUNCTION(mongo)
 	mongo_globals->chunk_size = DEFAULT_CHUNK_SIZE;
 	mongo_globals->cmd_char = "$";
 
-	mongo_globals->inc = 0;
 	mongo_globals->response_num = 0;
 	mongo_globals->errmsg = 0;
 
@@ -272,6 +278,7 @@ static PHP_GINIT_FUNCTION(mongo)
 	mongo_globals->machine = hash;
 
 	mongo_globals->ts_inc = 0;
+	mongo_globals->inc = rand() & 0xFFFFFF;
 
 #if PHP_VERSION_ID >= 50300
 	mongo_globals->log_callback_info = empty_fcall_info;
@@ -323,7 +330,6 @@ PHP_RINIT_FUNCTION(mongo)
 {
 	MonGlo(log_level) = 0;
 	MonGlo(log_module) = 0;
-	MonGlo(inc) = rand() & 0xFFFFFF;
 
 	return SUCCESS;
 }
@@ -340,8 +346,10 @@ PHP_MINFO_FUNCTION(mongo)
 	php_info_print_table_row(2, "Version", PHP_MONGO_VERSION);
 #if MONGO_PHP_STREAMS
 	php_info_print_table_row(2, "SSL Support", "enabled");
+	php_info_print_table_row(2, "Streams Support", "enabled");
 #else
 	php_info_print_table_row(2, "SSL Support", "disabled");
+	php_info_print_table_row(2, "Streams Support", "disabled");
 #endif
 
 	php_info_print_table_end();
