@@ -16,7 +16,7 @@
 #ifndef PHP_MONGO_H
 #define PHP_MONGO_H 1
 
-#define PHP_MONGO_VERSION "1.4.1"
+#define PHP_MONGO_VERSION "1.4.5"
 #define PHP_MONGO_EXTNAME "mongo"
 
 #ifdef HAVE_CONFIG_H
@@ -564,21 +564,21 @@ int php_mongo_set_readpreference(mongo_read_preference *rp, char *read_preferenc
 ZEND_BEGIN_MODULE_GLOBALS(mongo)
 	/* php.ini options */
 	char *default_host;
-	int default_port;
-	int request_id;
+	long default_port;
+	long request_id;
 	int chunk_size;
 
 	/* $ alternative */
 	char *cmd_char;
-	int native_long;
-	int long_as_object;
-	int allow_empty_keys;
+	long native_long;
+	long long_as_object;
+	long allow_empty_keys;
 
 	/* _id generation helpers */
 	int inc, pid, machine;
 
 	/* timestamp generation helper */
-	int ts_inc;
+	long ts_inc;
 	char *errmsg;
 	int response_num;
 	int pool_size;
@@ -614,7 +614,7 @@ extern zend_module_entry mongo_module_entry;
  * MongoException:
  * 0: The <class> object has not been correctly initialized by its constructor
  * 1: zero-length keys are not allowed, did you use $ with double quotes?
- * 2: '.' not allowed in key: <key>
+ * 2: characters not allowed in key: <key>
  * 3: insert too large: <size>, max: 16000000
  * 4: no elements in doc
  * 5: size of BSON doc is <size> bytes, max 4MB
