@@ -1,4 +1,4 @@
 #!/bin/bash
 
-export CFLAGS=-Wall
+export CFLAGS="-Wall -ggdb3"
 phpize && ./configure && make clean && make -j 5 all && make install
